package feedback;

/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is QueryUnit.java
 *
 * The Initial Developer of the Original Code is
 * Jinling Li, Freie Universit�t Berlin, ZE Botanischer Garten und Botanisches Museum
 * Berlin-Dahlem.
 *
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */

/**
 * <p>Title: QueryUnit</p>
 * <p>Description: class to send query to thread</p>
 * <p>Copyright: Copyright (c) 2005</p>
 * <p>Company: Botanic Garden and Botanical Museum Berlin-Dahlem</p>
 * @author Jinling Li
 * @version 1.0
 */

import org.apache.log4j.Level;
import org.apache.log4j.Logger;

public class QueryUnit {

	private Config conf;
	private String url;
	private String protocol;
	private String standard;
	private String dataSetId;
	private String sourceId;
	private String unitId;
	private String docStr = "";
	private Logger logger;

	public QueryUnit(Logger logger, String url, String protocol,
			String standard, String dataSetId, String sourceId, String unitId,
			Config config) {

		conf = config;
		this.url = url;
		this.protocol = protocol;
		this.standard = standard;
		this.dataSetId = dataSetId;
		this.sourceId = sourceId;
		this.unitId = unitId;
		this.logger = logger;

	}

	public String getDocStr() {
		return docStr;
	}

	// Access property
	public void setDocStr(String docStr, String rootElement) {
		if (docStr.length() > 0) {
			if(!protocol.equalsIgnoreCase("digir")) {
			int index = docStr.indexOf(">");
			docStr = docStr.substring(index + 1);
			}
			String header = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
			if(protocol.equalsIgnoreCase("digir")) {
				docStr = header +rootElement+ docStr;
			}else {
				docStr = header.trim() + rootElement.trim() + docStr;
			}
			this.docStr = docStr;
//			Text text = new Text(docStr);
//
//			Format f = Format.getPrettyFormat();
//			f.setTextMode(Format.TextMode.TRIM_FULL_WHITE);
//			f.setTextMode(Format.TextMode.NORMALIZE);
//			//f.setEncoding("ISO-8859-1");// "UTF-8");
//			f.setIndent("..");
//			XMLOutputter fmt = new XMLOutputter(f);
//			this.docStr = fmt.outputString(text);
//			try {
//				this.docStr = new String(this.docStr.getBytes("UTF-8"));
//			} catch (UnsupportedEncodingException e) {
//				// TODO Auto-generated catch block
//				e.printStackTrace();
//			}
		}
	}

	public String SendQuery() {
		String realQueryString = "";
		String rootElement = "";
		// database for query from digir.
		// String database = "";
		String database = "MammalsDwC2";

		if (protocol.equalsIgnoreCase("biocase")) {
			if (standard.equalsIgnoreCase("abcd1.2")) {
				realQueryString = conf.get12QueryString();
				rootElement = conf.getABCD12Root();

			} else if (standard.equalsIgnoreCase("abcd2.0")) {
				realQueryString = conf.get20QueryString();
				rootElement = conf.getABCD20Root();
			} else if (standard.equalsIgnoreCase("abcd2.05")) {
				realQueryString = conf.get205QueryString();
				rootElement = conf.getABCD205Root();
			} else if (standard.equalsIgnoreCase("abcd2.06")) {
				realQueryString = conf.get206QueryString();
				rootElement = conf.getABCD206Root();
			}

		} else if (protocol.equalsIgnoreCase("digir")) {
			if (standard.equalsIgnoreCase("darwincore")) {
				realQueryString = conf.getDarQueryString();
			} else if (standard.equalsIgnoreCase("darwincore")) {
				realQueryString = conf.getDarQueryString();
			} else if (standard.equalsIgnoreCase("darwincore")) {
				realQueryString = conf.getDarQueryString();
			}

			database = url.substring(0, url.indexOf("@"));
			url = url.substring(url.indexOf("@") + 1);
			realQueryString = conf.getDarQueryString();
			realQueryString = realQueryString.replaceFirst("__source__", conf
					.getSource());
			realQueryString = realQueryString.replaceFirst("__WrapperURL__",
					url);
			realQueryString = realQueryString.replaceFirst("__database__",
					database);

			rootElement = "";
		}

		boolean noDataSetId = false;
		boolean nosourceId = false;
		if (dataSetId.equals("")) {
			noDataSetId = true;
		}
		if (sourceId.equals("")) {
			nosourceId = true;
		}

		if (noDataSetId = true || nosourceId == true) {
			// realQueryString.
		}
		realQueryString = realQueryString.replaceFirst("__datasetId__",
				dataSetId);
		
		realQueryString = realQueryString
				.replaceFirst("__sourceId__", sourceId);
		realQueryString = realQueryString.replaceFirst("__unitId__", unitId);
		logger.info("The query is sent to data provider ");
		logger.debug("Query: \n"+realQueryString);
		try {
			Thread t = new UnitPoster(logger, realQueryString, url, 20000,
					conf, protocol, rootElement, this);
			t.start();
		} catch (Exception e) {
			// System.out.println("exception when loading URLs:");
			logger.log(Level.ERROR, "exception when loading URLs:");
			e.printStackTrace();
		}// try
		return realQueryString;

	}

}