package feedback;
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is XmlExtractor.java
 *
 * The Initial Developer of the Original Code is
 * Jinling Li, Freie Universit�t Berlin, ZE Botanischer Garten und Botanisches Museum
 * Berlin-Dahlem.
 *
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */

import java.io.CharArrayWriter;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import org.apache.log4j.Logger;
import org.apache.log4j.Level;


/**
 *
 * <p>Title: the XmlExtractor class</p>
 * <p>Description: extraction of information from BioCASE protocol comliant xml document.</p>
 * <p>August, 2005</p>
 * <p>Company: Botanic Garden and Botanical Museum Berlin-Dahlem</p>
 * @author Jinling Li
 * @version 1.0
 */

public class XmlExtractor extends DefaultHandler {

  private boolean incontents = false;             //flag to help us capture the elements inside element content.
  private CharArrayWriter textBuffer;             //buffer for putting extracted element data.
  private String ns = "";                         //the string of namespace uri.

  //some strings extracted from xml document.

  private String docStr = "";
  private String diagnostics = "";                // all the diagnostics as a stirng
  /**
   * Constructor
   */
  public XmlExtractor(Logger logger, Config conf, String protocol) {
    super();

    if(protocol.equalsIgnoreCase("biocase")) {
       ns = conf.getBioProtocolNsURI();
    }else if( protocol.equalsIgnoreCase("digir")) {
      ns = conf.getDiProtocolNsURI();
    }
    textBuffer = new CharArrayWriter();
    logger.log(Level.DEBUG, this.getClass().getName());
  }

  
/**
   * Extract a xml file
   * at the beginning of each new element, erase any text and
   * get some start elements information.
   */

  public void startElement(String namespaceURI, String localname,
                             String qname, Attributes attributes) throws SAXException {
    textBuffer.reset();
    if (namespaceURI.equals(ns)) {

      // get diagnostic attribute value if this attribute exist.
      if (localname.equals("diagnostic")) {
        if (attributes.getValue(0) != null) {
          diagnostics += "code=" + attributes.getValue(0) + ";";

        }
        if (attributes.getValue(1) != null) {
        diagnostics += "severity=" + attributes.getValue(1) + ";";

        }
      }
       if (localname.equals("content")) {
        incontents = true;

      }
    }
    if (incontents) {

      //get names and atrributes of the elements that inside content element.
      if (!localname.equals("content")) {

        String attr= "";
        if (localname.equalsIgnoreCase("record") ) {
        	attr ="   xmlns=\"http://digir.net/schema/protocol/2003/1.0\" xmlns:darwin=\"http://digir.net/schema/conceptual/darwin/2003/1.0\" xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xmlns:obis=\"http://www.iobis.org/obis\" xmlns:manis=\"http://digir.net/schema/protocol/2003/1.0\"";
        }
        for (int i = 0; i < attributes.getLength(); i++) {
            attr += " " + replaceInvalidChar(attributes.getLocalName(i)) + "=\"" + replaceInvalidChar(attributes.getValue(i)) + "\"" ;
        }
        	if(namespaceURI.equalsIgnoreCase("http://digir.net/schema/conceptual/darwin/2003/1.0"))
        	{
        		docStr += "<" + qname + attr + ">";
        		if(localname.equalsIgnoreCase("record")) {
        			docStr = docStr +"\n";
        		}
        	}else {
        		docStr += "<" + localname + attr + ">";
        	}

      }
    }
  }
  /**
   * take some action when we reach the end of selected elements.
   */

  public void endElement(String namespaceURI, String localname, String qname) throws SAXException {
    if (namespaceURI.equals(ns)) {

      //get value of diagnostic and append it to diagnostics string.
      if (localname.equals("diagnostic")) {
        diagnostics += "message=" + textBuffer.toString();
        diagnostics += "|";
      }
      // set flag for content.
     if (localname.equals("content")) {
            incontents = false;
      }
    }
    if (incontents && (!localname.equals("content"))) {
    	if(namespaceURI.equalsIgnoreCase("http://digir.net/schema/conceptual/darwin/2003/1.0"))
    	{
    		docStr += replaceInvalidChar(textBuffer.toString()) + "</" +qname + ">\n";
    	}else {
    	        docStr += replaceInvalidChar(textBuffer.toString()) + "</" + localname + ">\n";
    	}
        textBuffer.reset();

    }
  }
  // When the parser encounters plain text (not XML elements), it calls
  // this method, which accumulates them in a buffer.
  public void characters(char[] buffer, int start, int length) {
    textBuffer.write(buffer, start, length);
  }

  /**
   * report an exception
   */
  protected void report(SAXException exception) {
    System.out.println(exception.getMessage());

  }
  /**
   * Accessor for docStr.
   * @return docStr.
   */
  protected String getDocStr() {

     return docStr;

  }

  protected String getDiagnostics() {
    if(diagnostics != "") {
      int lastIndex = diagnostics.lastIndexOf("|");
      return diagnostics.substring(0, lastIndex);
    } else {
      //return diagnostics;
      return "diag-test";
    }
  }
  /**
   * Replace invalid xml character.
   * @return givenString
   */
  private String replaceInvalidChar(String givenString) {
    givenString = givenString.replaceAll("&", "&amp;");
    givenString = givenString.replaceAll(">", "&gt;");
    givenString = givenString.replaceAll("<", "&lt;");
    givenString = givenString.replaceAll("'", "&apos;");
    givenString = givenString.replaceAll("\"", "&quot;");
    return givenString;
  }
}// end of XmlExtractor
