package feedback;

/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Config.java
 *
 * The Initial Developer of the Original Code is
 * Jinling Li, Freie Universit�t Berlin, ZE Botanischer Garten und Botanisches Museum
 * Berlin-Dahlem.
 *
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */

import java.io.File;
import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.TimeZone;
import org.apache.log4j.Logger;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.input.SAXBuilder;

/**
 * <p> Title: Config </p>
 * <p> Description: get all configuration information from configuration file </p>
 * <p> August, 2005 </p>
 * <p> Company: Botanic Garden and Botanical Museum Berlin-Dahlem </p>
 * 
 * @author Jinling Li
 * @version 1.0
 */

public class Config {

	private String separator;
	private Element appRoot;
	private Logger logger;
	
	public Config(Logger logger) {
		this.logger=logger;
		// Filesystem
		separator = System.getProperty("file.separator");
		String confFile = "conf" + separator + "config.xml";

		try {

			URL url = feedback.Config.class.getResource(confFile);
			// read XML configuration files ...
			SAXBuilder builder = new SAXBuilder();

			Document AppConfigDoc = builder.build(new File(url.getFile()));
			appRoot = AppConfigDoc.getRootElement();
			logger.info("Config OK");

		} catch (Exception e) {
			logger.error(e.getMessage());
			//e.printStackTrace();
		}
	}

	/**
	 * returns the svn host name from config file
	 * @return string host name
	 */
	public String getSVNhost() {
		String host = "";
		try {
			host = appRoot.getChild("SVNhosturl").getTextTrim();
		} catch (Exception e) {
			logger.error(e.getMessage());
			//e.printStackTrace();
		}
		return host;
	}

	/**
	 * 
	 * @return svn user name from config file
	 */
	public String getSVNUsername() {
		String svnusername = "";
		try {
			svnusername = appRoot.getChild("SVNUsername").getTextTrim();
		} catch (Exception e) {
			logger.error(e.getMessage());
			//e.printStackTrace();
		}
		return svnusername;
	}
	
	/**
	 * 
	 * @return svn user password from config file
	 */
	public String getSVNPassword() {
		String svnpassword = "";
		try {
			svnpassword = appRoot.getChild("SVNPassword").getTextTrim();
		} catch (Exception e) {
			logger.error( e.getMessage());
			//e.printStackTrace();
		}
		return svnpassword;
	}
	/**
	 * 
	 * @return SVNfeedbackurl
	 */
	public String getSVNfeedbackurl() {
		String SVNfeedbackurl = "";
		try {
			SVNfeedbackurl = appRoot.getChild("SVNfeedbackurl").getTextTrim();
		} catch (Exception e) {
			logger.error( e.getMessage());
			//e.printStackTrace();
		}
		return SVNfeedbackurl;
	}
	
	/**
	 * 
	 * @return SVNfeedbacktrunk
	 */
	public String getSVNfeedbacktrunk() {
		String SVNfeedbacktrunk = "";
		try {
			SVNfeedbacktrunk = appRoot.getChild("SVNfeedbacktrunk").getTextTrim();
		} catch (Exception e) {
			logger.error( e.getMessage());
			//e.printStackTrace();
		}
		return SVNfeedbacktrunk;
	}
	
	/**
	 * 
	 * @return SVNfeedbackbranches
	 */
	public String getSVNfeedbackbranches() {
		String SVNfeedbackbranches = "";
		try {
			SVNfeedbackbranches = appRoot.getChild("SVNfeedbackbranches").getTextTrim();
		} catch (Exception e) {
			logger.error( e.getMessage());
			//e.printStackTrace();
		}
		return SVNfeedbackbranches;
	}
	
	public String getHost() {
		String host = "";
		try {
			host = appRoot.getChild("email").getChild("host").getTextTrim();

		} catch (Exception e) {
			logger.error( e.getMessage());
			//e.printStackTrace();
		}
		return host;
	}

	public String getRawDataDir() {
		String dir = "";
		try {
			dir = appRoot.getChild("rawDataDir").getTextTrim();

		} catch (Exception e) {
			logger.error( e.getMessage());
			//e.printStackTrace();
		}
		return dir;
	}

	public String getSource() {
		String source = "";
		try {
			source = appRoot.getChild("source").getTextTrim();

		} catch (Exception e) {
			logger.error( e.getMessage());
			//e.printStackTrace();
		}
		return source;
	}

	public String getSender() {
		String sender = "";
		try {
			sender = appRoot.getChild("email").getChild("sender").getTextTrim();

		} catch (Exception e) {
			logger.error( e.getMessage());
			//e.printStackTrace();
		}
		return sender;
	}

	public String getDarQueryString() {
		String finalQueryString = "";

		try {

			finalQueryString = appRoot.getChild("queryString").getChild(
					"dwcquery").getTextTrim();
			// finalQueryString
			finalQueryString = finalQueryString.replaceAll("__sendTime__",
					getDateAndTime());

		} catch (Exception e) {
			logger.error( e.getMessage());
			//e.printStackTrace();
		}
		return finalQueryString;
	}

	public String get12QueryString() {
		String finalQueryString = "";

		try {

			finalQueryString = appRoot.getChild("queryString").getChild(
					"abcd1.2query").getTextTrim();
			// finalQueryString
			finalQueryString = finalQueryString.replaceAll("__sendTime__",
					getDateAndTime());

		} catch (Exception e) {
			logger.error( e.getMessage());
			//e.printStackTrace();
		}
		return finalQueryString;
	}

	public String get20QueryString() {
		String finalQueryString = "";

		try {
			finalQueryString = appRoot.getChild("queryString").getChild(
					"abcd2.0query").getTextTrim();
			// finalQueryString
			finalQueryString = finalQueryString.replaceAll("__sendTime__",
					getDateAndTime());

		} catch (Exception e) {
			logger.error( e.getMessage());
			//e.printStackTrace();
		}
		return finalQueryString;
	}

	public String get205QueryString() {
		String finalQueryString = "";

		try {
			finalQueryString = appRoot.getChild("queryString").getChild(
					"abcd2.05query").getTextTrim();
			// finalQueryString
			finalQueryString = finalQueryString.replaceAll("__sendTime__",
					getDateAndTime());

		} catch (Exception e) {
			logger.error( e.getMessage());
			//e.printStackTrace();
		}
		return finalQueryString;
	}
	
	public String get206QueryString() {
		String finalQueryString = "";

		try {
			finalQueryString = appRoot.getChild("queryString").getChild(
					"abcd2.06query").getTextTrim();
			// finalQueryString
			finalQueryString = finalQueryString.replaceAll("__sendTime__",
					getDateAndTime());

		} catch (Exception e) {
			logger.error( e.getMessage());
			//e.printStackTrace();
		}
		return finalQueryString;
	}

	public String getDateAndTime() {
		String result;

		// get the time formatted as required
		java.util.Date now = new java.util.Date();
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd'T'hh:mm:ss");
		result = sdf.format(now);
		// and the timezone as required
		int time = TimeZone.getDefault().getRawOffset();
		int secs = time / 1000;
		int minutes = secs / 60;
		int hours = minutes / 60;
		if (hours >= 0) {
			result += "+";
		}
		if ((hours >= -9) && (hours <= 9)) {
			result += "0";
		}
		result += hours + ":00";

		return result;
	}

	public String getBioProtocolNsURI() {
		String result = "";
		// ... and read the parameter from xml document
		try {
			result = appRoot.getChild("protocolNs").getChild("biocase")
					.getChild("uri").getText();
		} catch (Exception e) {
			logger.error( e.getMessage());
			///e.printStackTrace();
			System.out
					.println("Exception when trying to read configuration file");
		}
		return result;
	}

	public String getDiProtocolNsURI() {
		String result = "";
		// ... and read the parameter from xml document
		try {
			result = appRoot.getChild("protocolNs").getChild("digir").getChild(
					"uri").getText();
		} catch (Exception e) {
			logger.error( e.getMessage());
			//e.printStackTrace();
			System.out
					.println("Exception when trying to read configuration file");
		}
		return result;
	}

	public String getABCD12Root() {
		String result = "";
		// ... and read the parameter from xml document
		try {
			result = appRoot.getChild("root").getChild("abcd1.2").getText();
		} catch (Exception e) {
			e.printStackTrace();
			System.out
					.println("Exception when trying to read configuration file");
		}
		return result;
	}

	public String getABCD205Root() {
		String result = "";
		// ... and read the parameter from xml document
		try {
			result = appRoot.getChild("root").getChild("abcd2.05").getText();
		} catch (Exception e) {
			logger.error( e.getMessage());
			//e.printStackTrace();
			System.out
					.println("Exception when trying to read configuration file");
		}
		return result;
	}
	public String getABCD206Root() {
		String result = "";
		// ... and read the parameter from xml document
		try {
			result = appRoot.getChild("root").getChild("abcd2.06").getText();
		} catch (Exception e) {
			//e.printStackTrace();
			logger.error( e.getMessage());
			System.out
					.println("Exception when trying to read configuration file");
		}
		return result;
	}
	
	public String getABCD20Root() {
		String result = "";
		// ... and read the parameter from xml document
		try {
			result = appRoot.getChild("root").getChild("abcd2.0").getText();
		} catch (Exception e) {
			logger.error( e.getMessage());
			//e.printStackTrace();
			System.out
					.println("Exception when trying to read configuration file");
		}
		return result;
	}

	public String get12EmailPath() {
		String result = "";
		// ... and read the parameter from xml document
		try {
			result = appRoot.getChild("providerEmailPath").getChild("abcd1.2")
					.getText();
		} catch (Exception e) {
			logger.error( e.getMessage());
			//e.printStackTrace();
			System.out
					.println("Exception when trying to read configuration file");
		}
		return result;
	}

	public String get20EmailPath() {
		String result = "";
		// ... and read the parameter from xml document
		try {
			result = appRoot.getChild("providerEmailPath").getChild("abcd2.0")
					.getText();
		} catch (Exception e) {
			logger.error( e.getMessage());
			//e.printStackTrace();
			System.out
					.println("Exception when trying to read configuration file");
		}
		return result;
	}
	
	public String getDarwinEmailPath() {
		String result = "";
		// ... and read the parameter from xml document
		try {
			result = appRoot.getChild("providerEmailPath").getChild("darwin")
					.getText();
		} catch (Exception e) {
			logger.error( e.getMessage());
			//e.printStackTrace();
			System.out
					.println("Exception when trying to read configuration file");
		}
		return result;
	}

	public String get12NS() {
		String result = "";
		// ... and read the parameter from xml document
		try {
			result = appRoot.getChild("abcdNS").getChild("abcd1.2").getText();
		} catch (Exception e) {
			//e.printStackTrace();
			logger.error("Exception when trying to read configuration file");
			
		}
		return result;
	}

	public String get20NS() {
		String result = "";
		// ... and read the parameter from xml document
		try {
			result = appRoot.getChild("abcdNS").getChild("abcd2.0").getText();
		} catch (Exception e) {
			logger.error( e.getMessage());
			//e.printStackTrace();
			logger.error("Exception when trying to read configuration file");
		}
		return result;
	}
	public String get205NS() {
		String result = "";
		// ... and read the parameter from xml document
		try {
			result = appRoot.getChild("abcdNS").getChild("abcd2.05").getText();
		} catch (Exception e) {
			//e.printStackTrace();
			logger.error( e.getMessage());
			logger.error("Exception when trying to read configuration file");
		}
		return result;
	}

	public String get206NS() {
		String result = "";
		// ... and read the parameter from xml document
		try {
			result = appRoot.getChild("abcdNS").getChild("abcd2.06").getText();
		} catch (Exception e) {
			logger.error( e.getMessage());
			//e.printStackTrace();
			logger.error("Exception when trying to read configuration file");
		}
		return result;
	}
	
	public String getdigirNS() {
		String result = "";
		// ... and read the parameter from xml document
		try {
			result = appRoot.getChild("digirNS").getChild("digir").getText();
		} catch (Exception e) {
			e.printStackTrace();
			logger.error("Exception when trying to read configuration file");
		}
		return result;
	}
	public String getdigirmanisNS() {
		String result = "";
		// ... and read the parameter from xml document
		try {
			result = appRoot.getChild("digirNS").getChild("digir_manis").getText();
		} catch (Exception e) {
			e.printStackTrace();
			logger.error("Exception when trying to read configuration file");
		}
		return result;
	}

	public String getdigirobisNS() {
		String result = "";
		// ... and read the parameter from xml document
		try {
			result = appRoot.getChild("digirNS").getChild("digir_obis").getText();
		} catch (Exception e) {
			e.printStackTrace();
			logger.error("Exception when trying to read configuration file");
		}
		return result;
	}

	public String getAnnotationEmailSubject()
	{
		String result = "";
		// ... and read the parameter from xml document
		try {
			result = appRoot.getChild("AnnotationEmail").getChild("subject").getText();
		} catch (Exception e) {
			e.printStackTrace();
			logger.error("Exception when trying to read configuration file");
		}
		return result;
	}
	public String getAnnotationEmailfrom()
	{
		String result = "";
		// ... and read the parameter from xml document
		try {
			result = appRoot.getChild("AnnotationEmail").getChild("from").getText();
		} catch (Exception e) {
			e.printStackTrace();
			logger.error("Exception when trying to read configuration file");
		}
		return result;
	}
	
	public String getAnnotationEmailContent()
	{
		String result = "";
		// ... and read the parameter from xml document
		try {
			result = appRoot.getChild("AnnotationEmail").getChild("content").getText();
		} catch (Exception e) {
			e.printStackTrace();
			logger.error("Exception when trying to read configuration file");
		}
		return result;
	}
}
