package feedback;

/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is DealUserInfo.java
 *
 * The Initial Developer of the Original Code is
 * Jinling Li, Freie Universit�t Berlin, ZE Botanischer Garten und Botanisches Museum
 * Berlin-Dahlem.
 *
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */

import java.io.Serializable;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Types;
import java.util.Properties;

import javax.mail.Address;
import javax.mail.Message;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;

import org.apache.log4j.Logger;

/**
 * <p> Title: DealUserInfo </p>
 * <p> Description: Deal all the business related user account </p>
 * <p> August, 2005 </p>
 * <p> Company: Botanic Garden and Botanical Museum Berlin-Dahlem </p>
 * @author Jinling Li
 * @version 1.0
 */
public class DealUserInfo implements Serializable {

	/**
	 * 
	 */
	private static final long serialVersionUID = -2778211027390318380L;
	private Connection dbConn;
	private Config conf;
	private Properties props;
	private String host;
	private String from;
	private Session mailSession;
	private Message msg;
	private Logger logger;

	/** Creates a new instance of UserInfo */
	public DealUserInfo(Logger logger2, Config config) {

		this.conf = config;
		host = conf.getHost();
		from = conf.getSender();
		this.logger = logger2;

	}

	public void setConn(Connection dbConn) {
		this.dbConn = dbConn;
	}

	public Config getConf() {
		return conf;
	}

	// login to the DB
	public synchronized int isOldUser(String email, String password) {
		int userid = -1;

		try {
			String sql = "SELECT user_id FROM user_registry WHERE email = '"
					+ email + "' AND password =  '" + password + "'";
			Statement st = dbConn.createStatement();
			ResultSet rs = st.executeQuery(sql);
			if (rs.next() == true) {
				userid = rs.getInt("user_id");
			}
			rs.close();
			rs = null;
			st.close();
			st = null;
			logger.info("User already exist!");

		} catch (SQLException e) {
			e.printStackTrace();
			logger.error("error when check isOldUser!");
			userid = -1;
		}
		return userid;
	}

	public synchronized String getPass(String email) {
		String password = "";
		try {
			String sql = "SELECT password FROM user_registry WHERE email = '"
					+ email + "'";
			Statement st = null;
			st = dbConn.createStatement();
			ResultSet rs = null;
			rs = st.executeQuery(sql);
			if (rs.next() == true) {
				password = rs.getString("password");
			}
			rs.close();
			rs = null;
			st.close();
			st = null;
			logger.info("Email already exist!");
		} catch (SQLException e) {
			e.printStackTrace();
			logger.error("error when get OldPass!");
			//System.err.println("error when get OldPass!");
			password = "";
		}
		return password;
	}

	public synchronized String getEmail(int userid) {
		String email = "";
		try {
			String sql = "SELECT email FROM user_registry WHERE user_id = '"
					+ userid + "'";
			Statement st = null;
			st = dbConn.createStatement();
			ResultSet rs = null;
			rs = st.executeQuery(sql);
			if (rs.next() == true) {
				email = rs.getString("email");
			}
			rs.close();
			rs = null;
			st.close();
			st = null;
			logger.info("User found");
		} catch (SQLException e) {
			e.printStackTrace();
			logger.error("error when get user email!");
			//System.err.println("error when get OldPass!");
			email = "";
		}
		return email;
	}
	
	
	public synchronized void changePass(String pass, String email) {
		try {
			String sql = "Update user_registry SET password = '" + pass
					+ "'WHERE email = '" + email + "'";
			Statement st = null;
			st = dbConn.createStatement();
			st.executeQuery(sql);
			st.close();
			st = null;
			logger.info("Password changed!");
		} catch (SQLException e) {
			e.printStackTrace();
			logger.error("error when change password!");
		}
	}

	public void sendRegEmail(String email, String confirmUrl) {
		String subject = "Registry for annotation";
		String content = "Hello,<br>"
				+ "You have registered to make an annotation. Please confirm your request by clicking on the link below.<br>"
				+ "This confirmaiton step helps us to avoid false account creation requests and inaccurate email contact information.<br><br>"
				+ "<a href=" + confirmUrl
				+ ">Click here to verify your registration</a> <br><br>"
				+ "thank you!";
		sendEmail(email, subject, content);
		logger.info("an email has been sent to the user for confirming registration!");
	}

	public void sendPassEmail(String email, String password) {
		String subject = "Your Password for Annotation";
		String content = "Hello,<br><br>" + "Your password is: " + password
				+ "<br><br>" + "Please use this password to login!<br><br>"
				+ "thank you!";
		sendEmail(email, subject, content);
		logger.info("an email has been sent to the user for reminding password!");
	}

	private void sendEmail(String email, String subject, String content) {
		try {
			Properties props = new Properties();
			// set the host for sending email
			props.put("mail.host", host);
			Session mailConnection = Session.getInstance(props);
			Message msg = new MimeMessage(mailConnection);
			Address biocase = new InternetAddress(from, "Feedback");
			Address recipient = new InternetAddress(email);
			msg.setFrom(biocase);
			msg.setRecipient(Message.RecipientType.TO, recipient);
			msg.setSubject(subject);
			msg.setContent(content, "text/html");
			Transport.send(msg);
			logger.info("finished to send email to " + email);

		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	// registry to the DB
	public int registry(String name, String password, String email) {
		int userid = nextFreeId();
		String sql = "INSERT into user_registry(user_id, name, password, email) values (?, ?, ?, ?)";
		try {
			PreparedStatement pstmt = null;
			pstmt = dbConn.prepareStatement(sql);
			pstmt.setInt(1, userid);
			if (name.equals("")) {
				pstmt.setNull(2, Types.VARCHAR);
			} else {
				pstmt.setString(2, name);
			}
			if (password.equals("")) {
				pstmt.setNull(3, Types.VARCHAR);
			} else {
				pstmt.setString(3, password);
			}

			if (email.equals("")) {
				pstmt.setNull(4, Types.VARCHAR);
			} else {
				pstmt.setString(4, email);
			}
			pstmt.executeUpdate();
			pstmt.close();
			pstmt = null;
			logger.info("a new user has been registered!");
		} catch (SQLException e) {
			logger.error("SQL exception when inserting a new user!");
			e.printStackTrace();
		}

		return userid;
	}

	// Return the first unoccupied identifier (integer) in the given table.
	private int nextFreeId() {
		int newId = -1;
		String theQuery = "SELECT newuserid()";
		try {
			Statement stmt1 = null;
			stmt1 = dbConn.createStatement();
			ResultSet idRs = null;
			idRs = stmt1.executeQuery(theQuery);
			boolean dummy = idRs.next();
			newId = idRs.getInt(1);
			logger.info("New user id from sql function: "+newId);
			stmt1.close();
			stmt1 = null;
			idRs.close();
			idRs = null;
		} catch (SQLException e) {
			logger.error("Exception when trying to generate a new id for table: ");
			e.printStackTrace();
		} 
		return newId;
	}
}
