package feedback;

/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is FeedbackController.java
 *
 * The Initial Developer of the Original Code is
 * Jinling Li, Freie Universit�t Berlin, ZE Botanischer Garten und Botanisches Museum
 * Berlin-Dahlem.
 *
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */

/**
 * <p>
 * Title: FeedbackController
 * </p>
 * <p>
 * Description: class to control all the business of the feedback
 * </p>
 * <p>
 * August, 2005
 * </p>
 * <p>
 * Company: Botanic Garden and Botanical Museum Berlin-Dahlem
 * </p>
 * 
 * @author Jinling Li
 * @version 1.0
 */

import java.io.IOException;
import java.net.URLEncoder;
import java.sql.Connection;
import java.sql.SQLException;
import java.text.SimpleDateFormat;
import java.util.logging.LogRecord;
import java.util.logging.SimpleFormatter;

import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import javax.sql.DataSource;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;


public class FeedbackController extends HttpServlet {

	/**
	 * 
	 */
	private static final long serialVersionUID = -2987766861826716772L;
	private String url;
	private String protocol;
	private String standard;
	private String datasetId;
	private String sourceId;
	private String unitId;
	private Connection dbConn = null;
	private Connection dbConnCorm = null;
	private CormInfo cormInfo;
	private DealUserInfo uinfo = null;
	private int userid;
	private String uddi;
	private Logger logger;
	private String oldStr = "";
	private long revision = 0;
	private Config config;
	private ProcessResult pr;
	private String tomcatPath;

	/**
	 * Create DealUserInfo, log file and DB connection
	 */
	public void init(ServletConfig servletConfig) throws ServletException {

		// create log file
		try {
			 tomcatPath = servletConfig.getServletContext().getRealPath(
					"");
			String logconfigfilename = servletConfig
					.getInitParameter("log_config_file");
			PropertyConfigurator.configureAndWatch(tomcatPath + "/"
					+ logconfigfilename);

			logger = Logger.getLogger(this.getClass());
			logger.info("FeedBack system starting....");
			DataSource source = null;
			source = (DataSource) new InitialContext()
					.lookup("java:comp/env/jdbc/feedback");
			dbConn = source.getConnection();
			logger.info("Annotation Database has been connected.");
			source = (DataSource) new InitialContext()
					.lookup("java:comp/env/jdbc/corm");
			dbConnCorm = source.getConnection();
			logger.info("CORM Database has been connected.");

			config = new Config(logger);
			/**
			 * Creates a DealUserInfo, and saves it as servlet context
			 * attribute.
			 */
			uinfo = new DealUserInfo(logger, config);
			uinfo.setConn(dbConn);
			// getServletContext().setAttribute("uinfo", uinfo);
			/**
			 * Creates a CormInfo.
			 */
			this.cormInfo = new CormInfo(logger, config);
			cormInfo.setConn(dbConnCorm);
			
		} catch (SQLException e) {
			e.printStackTrace();
			logger.error("SQLException: " + e);
		} catch (NamingException e) {
			e.printStackTrace();
			logger.error("NamingException: " + e);
			config = new Config(logger);
		}
	}

	/**
	 * Remove the DealUserInfo, colse DB and logger.
	 * 
	 */
	public void destroy() {
		getServletContext().removeAttribute("uinfo");

		if (dbConn != null) {
			try {
				dbConn.close();
				logger.info("Annotation DB Connection closed");
			} catch (SQLException e) {
			}
			dbConn = null;
		}
		if (dbConnCorm != null) {
			try {
				dbConnCorm.close();
				logger.info("Corm DB Connection closed");
			} catch (SQLException e) {
			}
			dbConnCorm = null;
		}
		logger.info("Servlet is destroyed");
		// Close logger
		logger.info("Annotation process finished.");
	}

	/**
	 * Processe a GET request by calling doPost().
	 */
	public void doGet(HttpServletRequest request, HttpServletResponse response)
			throws IOException, ServletException {
		doPost(request, response);
	}

	/**
	 * Perform authentication, if needed, and access control. If the user is
	 * authorized, perform the action specified by the "action" request
	 * parameter.
	 */
	public void doPost(HttpServletRequest request, HttpServletResponse response)
			throws IOException, ServletException {

		response.setCharacterEncoding("UTF-8");

		String action = request.getParameter("action");
		// Check if the user is authenticated
		if (!isAuthenticated(request) && !("authenticate".equals(action)) &&
		// this action from the login page
				!("registry".equals(action)) && // this action from the registry
				// page
				!("confirm".equals(action)) && // this action from the confirm
				// page
				!("remindpass".equals(action)) && // this action from the
				// remindpass page
				!("toLogin".equals(action))) { // this action from the record
			// page
			doRedirectToLogin(request, response);
		} else {
			if ("toLogin".equals(action)) {
				// pass all the six parameters from record page and ten go to
				// the login page.
				url = request.getParameter("url");
				if (url == null) {
					throw new ServletException("Missing url");
				}

				protocol = request.getParameter("protocol");
				if (protocol == null) {
					throw new ServletException("Missing protocol");
				}
				
				datasetId = new String(request.getParameter("datasetId").toString().getBytes("ISO-8859-1"),"UTF-8");
				sourceId = new String(request.getParameter("sourceId").toString().getBytes("ISO-8859-1"),"UTF-8");
				logger.info("Dataset : " + datasetId +" Sourceid : "+sourceId);
				unitId = request.getParameter("unitId");
				if (unitId == null) {
					throw new ServletException("Missing unitId");
				}

				standard = request.getParameter("standard");
				if (standard == null) {
					throw new ServletException("Missing standard");
				} else if (standard.startsWith("http") == true) {
					try {
						standard = simplify(standard);
					} catch (Exception e) {
						logger
								.info("Unrecognied namespace check standard parameter !! "
										+ standard);
					}
				}
				doToLogin(request, response);
			} else if ("authenticate".equals(action)) {
				doAuthenticate(request, response);
			} else if ("registry".equals(action)) {
				doRegistry(request, response);
			} else if ("confirm".equals(action)) {
				doConfirm(request, response);
			}
			else if ("edit".equals(action)) {
				doEdit(request, response);
			} else if ("remindpass".equals(action)) {
				doRemindPass(request, response);
			} else if ("changepass".equals(action)) {
				doChangePass(request, response);
			} else {
				response.sendError(HttpServletResponse.SC_NOT_IMPLEMENTED);
			}
		}
	}

	/**
	 * Returns the short name of the standard
	 */
	private String simplify(String stand) throws Exception {
		String standard = null;
		if (stand.equalsIgnoreCase(config.get12NS()) == true
				|| stand.equalsIgnoreCase("abcd1.2")) {
			return standard = "abcd1.2";
		}
		if (stand.equalsIgnoreCase(config.get205NS()) == true) {
			return standard = "abcd2.05";
		}
		if (stand.equalsIgnoreCase(config.get206NS()) == true) {
			return standard = "abcd2.06";
		}
		if (stand.equalsIgnoreCase("abcd2.0") == true) {
			return standard = stand;
		}
		if (stand.equalsIgnoreCase(config.getdigirNS()) == true) {
			return standard = "digir";
		}
		if (stand.equalsIgnoreCase("digir") == true) {
			return standard = stand;
		}
		if (stand.equalsIgnoreCase(config.getdigirmanisNS()) == true) {
			return standard = "digir_manis";
		}
		if (stand.equalsIgnoreCase(config.getdigirobisNS()) == true) {
			return standard = "digir_obis";
		}
		if (standard == null) {
			throw new Exception("Namespace unregnozied");
		}
		return standard;

	}

	/**
	 * Returns true if an authentication token object is found in the session.
	 */
	private boolean isAuthenticated(HttpServletRequest request) {
		boolean isAuthenticated = false;
		HttpSession session = request.getSession();
		if (session.getAttribute("validUser") != null) {
			isAuthenticated = true;
		}
		return isAuthenticated;
	}

	/**
	 * Forwards the request to the login page, with an "errorMsg" parameter
	 * containing text to be displayed on the login page.
	 */
	private void doRedirectToLogin(HttpServletRequest request,
			HttpServletResponse response) throws IOException, ServletException {

		String loginURL = "login.jsp" + "?errorMsg="
				+ URLEncoder.encode("Please log in first", "UTF-8");

		response.sendRedirect(loginURL);
	}

	private void doToLogin(HttpServletRequest request,
			HttpServletResponse response) throws IOException, ServletException {

		HttpSession session = request.getSession();
		String loginURL = "login.jsp";
		session.setAttribute("notes", "");
		response.sendRedirect(loginURL);
	}

	/**
	 * Authenticates a user using the "email" and "password" request parameters.
	 * 
	 */
	private void doAuthenticate(HttpServletRequest request,
			HttpServletResponse response) throws IOException, ServletException {

		final String apply = "Login";
		final String cancel = "Cancel";
		String value = request.getParameter("dobtn");

		if (apply.equals(value)) {

			String email = request.getParameter("email");
			if (email == null) {
				throw new ServletException("Missing email");
			}
			String password = request.getParameter("password");
			HttpSession session = request.getSession();
			if (password == null) {
				throw new ServletException("Missing Password");
			}
			try {
				userid = this.uinfo.isOldUser(email, password);
				try {
					logger.info("URL" +url + "; protocol" +protocol);
					uddi = cormInfo.getCormUDDI(url, protocol).trim();
				} catch (Exception e) {
					session.setAttribute("Msg", "No UDDI key found for your provider.");
					logger
							.info("No UDDI Key found in the CORM DB !! \n Please check your url !!");
					throw e;
				}
				// if successfully login, then process unit record and
				// go to editor page.
				if (userid > 0) {
					logger.info("User login successfully!");
					
					session.setAttribute("validUser", userid + "");
					String id = session.getId();
					session.setAttribute("email", email);
					// process unit result.
					this.pr = new ProcessResult(logger, config, uddi, url,
							protocol, standard, datasetId, sourceId, unitId,
							userid);
					 String  docStr = pr.getIntermediate();
					 if (docStr.indexOf("__tomcatpath__")>0){
						 docStr = docStr.replaceAll("__tomcatpath__", tomcatPath);
					 }
					 
					//session.setAttribute("origxmlDoc", replaceInvalidChar(docStr));
					//session.setAttribute("xmlDoc", replaceInvalidChar(docStr));
					 session.setAttribute("origxmlDoc", docStr);
					 session.setAttribute("xmlDoc", docStr);
					session.setAttribute("protocol", protocol);
					session.setAttribute("Msg", "");
					// Redirect to editor page
					String next = request.getParameter("origURL");
					if (next == null || next.length() == 0) {
						next = next + "editor.jsp" + "?userid="
								+ URLEncoder.encode(id, "UTF-8");
					}
					response.sendRedirect(next);

				}
				// redirect to login if invalid user
				else {
					String loginURL = "login.jsp"
							+ "?errorMsg="
							+ URLEncoder.encode("Invalid Email or Password",
									"UTF-8");
					response.sendRedirect(loginURL);
				}
			} catch (Exception e) {
				logger.error("Servlet exception:" + e);
			}
		} else if (cancel.equals(value)) {

			response.sendRedirect("login.jsp");
		}

	}

	private void doRegistry(HttpServletRequest request,
			HttpServletResponse response) throws IOException, ServletException {

		final String confirm = "Register";
		final String cancel = "Cancel";
		String value = request.getParameter("dobtn");

		String name = request.getParameter("name");
		if (name == null) {
			throw new ServletException("Missing User Name");
		}
		String password = request.getParameter("password");
		if (password == null) {
			throw new ServletException("Missing Password");
		}
		String email = request.getParameter("email");
		if (email == null) {
			throw new ServletException("Missing email");
		}
		HttpSession session = request.getSession();
		// session id
		String id = session.getId();
		if (confirm.equals(value)) {
			if (name.length() != 0 & password.length() != 0
					& email.length() != 0) {
				try {
					// check if the user has an account already
					if (uinfo.getPass(email).length() > 0) {
						String registryURL = "registry.jsp"
								+ "?registryMsg="
								+ URLEncoder
										.encode(
												"You have already registered with this email. Please go to login page by click Cancel button",
												"UTF-8");
						response.sendRedirect(registryURL);
					}
					// if new user, send email to the user for confirmation
					else {

						session.setAttribute("name", name);
						session.setAttribute("password", password);
						session.setAttribute("email", email);

						String origURL = request.getRequestURL().toString()
								.replaceFirst("process", "");
						String confirmUrl = origURL + "confirm.jsp"
								+ "?userid=" + URLEncoder.encode(id, "UTF-8");

						uinfo.sendRegEmail(email, confirmUrl);

						// Redirect to a info page
						String next = "";

						next = request.getRequestURL().toString().replaceFirst(
								"process", "")
								+ "regInfo.jsp"
								+ "?userid="
								+ URLEncoder.encode(id, "UTF-8");

						response.sendRedirect(next);
					}
				} catch (Exception e) {
					throw new ServletException("error", e);
				}
			}

			// redirect to the registry page if some of the fields are not
			// filled
			else {
				String registryURL = "registry.jsp"
						+ "?registryMsg="
						+ URLEncoder.encode("Please Fill All the Information",
								"UTF-8");
				response.sendRedirect(registryURL);

			}
		} else if (cancel.equals(value)) {

			response.sendRedirect("login.jsp");

		}

	}

	private void doRemindPass(HttpServletRequest request,
			HttpServletResponse response) throws IOException, ServletException {

		final String confirm = "Confirm";
		final String cancel = "Cancel";
		String value = request.getParameter("dobtn");
		if (confirm.equals(value)) {
			String email = request.getParameter("email");
			if (email.length() > 0) {
				String password = uinfo.getPass(email);
				if (password.length() > 0) {
					uinfo.sendPassEmail(email, password);
					response.sendRedirect("login.jsp");
				} else {
					String loginURL = "remindpass.jsp"
							+ "?Msg="
							+ URLEncoder.encode(
									"You have not registered with this email!",
									"UTF-8");
					response.sendRedirect(loginURL);

				}
			}
		} else if (cancel.equals(value)) {

			response.sendRedirect("login.jsp");

		}

	}

	private void doChangePass(HttpServletRequest request,
			HttpServletResponse response) throws IOException, ServletException {

		final String apply = "Apply";

		String value = request.getParameter("dobtn");

		if (apply.equals(value)) {
			HttpSession session = request.getSession();
			String email = (String) session.getAttribute("email");
			if (email == null) {
				throw new ServletException("Missing email");
			}
			String newpassword = request.getParameter("newpassword");
			if (newpassword == null) {
				throw new ServletException("Missing NewPassword");
			}
			String renewpassword = request.getParameter("renewpassword");
			if (renewpassword == null) {
				throw new ServletException("Missing RenewPassword");
			}

			String url = "";
			String id = session.getId();

			url = request.getRequestURL().toString()
					.replaceFirst("process", "")
					+ "editor.jsp"
					+ "?userid="
					+ URLEncoder.encode(id, "UTF-8");

			if (newpassword.matches(renewpassword)
					&& renewpassword.length() > 0) {
				try {
					DealUserInfo uinfo = (DealUserInfo) getServletContext()
							.getAttribute("uinfo");

					uinfo.changePass(newpassword, email);
					String docStr = session.getAttribute("xmlDoc").toString();
					session.setAttribute("xmlDoc", replaceInvalidChar(docStr));
					response.sendRedirect(url);

				} catch (Exception e) {
					throw new ServletException("error", e);
				}
			} else {
				String toUrl = request.getRequestURL().toString().replaceFirst(
						"process", "")
						+ "changepass.jsp"
						+ "?userid="
						+ URLEncoder.encode(id, "UTF-8")
						+
						"&Msg="
						+ URLEncoder.encode("Please verify your password",
								"UTF-8");
				response.sendRedirect(toUrl);
			}
		}
	}

	private void doConfirm(HttpServletRequest request,
			HttpServletResponse response) throws IOException, ServletException {

		final String confirm = "Confirm";
		final String cancel = "Cancel";
		String value = request.getParameter("dobtn");
		String email = "";

		if (confirm.equals(value)) {

			HttpSession session = request.getSession();
			String name = session.getAttribute("name").toString();
			String password = session.getAttribute("password").toString();

			// force page to login page if use back button to make confirm again
			// or make confirm from email
			if (session.getAttribute("email") == null
					|| (session.getAttribute("email") != null)
					&& (uinfo.isOldUser(email, password) > 0)) {
				doRedirectToLogin(request, response);
			} else {
				email = session.getAttribute("email").toString();
				try {
					userid = uinfo.registry(name, password, email);
					if (userid > 0) {
						try {
							logger.info("URL" +url + "; protocol" +protocol);
							uddi = cormInfo.getCormUDDI(url, protocol).trim();
						} catch (Exception e) {
							logger
									.info("No UDDI Key found in the CORM DB !! \n Please check your url !!");
							throw e;
						}
						session.setAttribute("validUser", userid + "");	
						this.pr = new ProcessResult(logger, config, uddi, url,
								protocol, standard, datasetId, sourceId, unitId,
								userid);
						logger.info("pr created");
						String docStr = this.pr.getIntermediate();
						logger.info("xml intermediat");
						logger.debug(docStr);
						session.setAttribute("origxmlDoc", replaceInvalidChar(docStr));
						session.setAttribute("xmlDoc", replaceInvalidChar(docStr));
						session.setAttribute("protocol", protocol);
						String next = "";
						String id = session.getId();
						next = request.getRequestURL().toString().replaceFirst(
								"process", "")
								+ "editor.jsp"
								+ "?userid="
								+ URLEncoder.encode(id, "UTF-8");
						response.sendRedirect(next);
						if (docStr.length() > 0) {
							//revision = pr.processSVN(true, docStr, -1, "");
							// keep the old record string for next step to read
							// the provider email
							oldStr = docStr;
						}

					}// if
				} catch (Exception e) {
					// throw new ServletException("error", e);
					logger.error("Servlet exception:" + e);
				}// try
			}
		} else if (cancel.equals(value)) {
			response.sendRedirect("login.jsp");
		}

	}

	private void doEdit(HttpServletRequest request, HttpServletResponse response)
			throws IOException, ServletException {

		final String apply = "Apply";
		final String validate = "Check";
		final String cancel = "Cancel";
		final String logout = "Logout";
		final String changepass = "Change your password";

		String value = request.getParameter("dobtn");
		HttpSession session = request.getSession();

		String id = session.getId();
		//session.setAttribute("xmlDoc", request.getParameter("xmlDoc"));
		//String docStr = session.getAttribute("xmlDoc").toString();
		String docStr = new String(request.getParameter("xmlDoc").getBytes("ISO-8859-1"),"UTF-8");
		
		session.setAttribute("email", session.getAttribute("email").toString());
		String vmessage ="";
		if (apply.equals(value)) {
			oldStr = session.getAttribute("origxmlDoc").toString();
			oldStr = replaceInvalidCharBack(oldStr);
			//oldStr = oldStr.replaceAll("&", "&amp;");
			docStr = docStr.replaceAll("&", "&amp;");
			//session.setAttribute("notes", request.getParameter("notes"));
			String notesStr = new String(request.getParameter("notes").toString().getBytes("ISO-8859-1"),"UTF-8");
			// ProcessResult pr = new ProcessResult(logger, userid, url,
			// datasetId, sourceId, unitId);
			
			// edit page for biocase protocol
			if (protocol.equalsIgnoreCase("biocase")) {

				//DealUserInfo uinfo = (DealUserInfo) getServletContext()
				//		.getAttribute("uinfo");

				if (protocol.equalsIgnoreCase("biocase")) {
					Validator vd = new Validator(logger);
					docStr = docStr.replaceAll("&amp;", "&");
					docStr = docStr.replaceAll("&", "&amp;");
					vd.validate(docStr);
					vmessage = vd.getMessage();
					docStr = replaceInvalidCharBack(docStr);
					session.setAttribute("VMessage", vmessage);
					session.setAttribute("xmlDoc", docStr);
					session.setAttribute("notes", new String(request.getParameter("notes").toString().getBytes("ISO-8859-1"),"UTF-8"));

				}
				// if the xml file is valid, then process the annotation
				if (session.getAttribute("VMessage") != null) { // "VMessage" is
					// the message
					// from
					// validating
					if (docStr.startsWith("<?xml")
							& (session.getAttribute("VMessage").toString()
									.equalsIgnoreCase("The edited XML file is valid!"))) {

						// put the original xml file intto SVN
						revision = pr.processSVN(true, oldStr, -1, "");
						pr.setOrigrevision(revision);
						pr.createAnnotation(standard, revision, notesStr,
								config, oldStr, docStr, dbConn,uinfo.getEmail(this.userid));
						
						// kill the valid user session and email, then redirect to record
						// page.
						if (session.getAttribute("validUser") != null) {
							session.setAttribute("validUser", null);
						}
						if (session.getAttribute("email") != null) {
							session.setAttribute("email", null);
						}
						response.sendRedirect("success.jsp");
					}
					// xml file is not valid
					else {
						// pr.createAnnotation(standard, revision, notesStr,
						// config, oldStr, docStr, dbConn);
						String loginURL = "editor.jsp?userid="
							+ URLEncoder.encode(id, "UTF-8") + "&Msg="
							+ URLEncoder.encode(vmessage, "UTF-8");
					response.sendRedirect(loginURL);
					} // if
				}
				// if have not perform validate
				else {
					revision = pr.processSVN(true, oldStr, -1, "");
					pr.setOrigrevision(revision);
					// keep the old record string for next step to read the

					pr.createAnnotation(standard, revision, notesStr, config,
							oldStr, docStr, dbConn,uinfo.getEmail(this.userid));
					// kill the valid user session and email, then redirect to record
					// page.
					if (session.getAttribute("validUser") != null) {
						session.setAttribute("validUser", null);
					}
					if (session.getAttribute("email") != null) {
						session.setAttribute("email", null);
					}
					response.sendRedirect("success.jsp");
				}
			} else { // for digir protocol, presently, there is no need to
				// validate the xml file
				if (docStr.startsWith("<?xml")) { // if get xml record
					// put the original xml file intto SVN
					revision = pr.processSVN(true, oldStr, -1, "");
					pr.setOrigrevision(revision);
					pr.createAnnotation(standard, revision, notesStr, config, oldStr,
							docStr, dbConn,uinfo.getEmail(this.userid));
					// kill the valid user session and email, then redirect to record
					// page.
					if (session.getAttribute("validUser") != null) {
						session.setAttribute("validUser", null);
					}
					if (session.getAttribute("email") != null) {
						session.setAttribute("email", null);
					}
					response.sendRedirect("success.jsp");
				} else {
					revision = pr.processSVN(true, oldStr, -1, "");
					pr.setOrigrevision(revision);
					pr.createAnnotation(standard, revision, notesStr, null, "",
							"", dbConn,uinfo.getEmail(this.userid));
					// kill the valid user session and email, then redirect to record
					// page.
					if (session.getAttribute("validUser") != null) {
						session.setAttribute("validUser", null);
					}
					if (session.getAttribute("email") != null) {
						session.setAttribute("email", null);
					}
					response.sendRedirect("success.jsp");
				} // if
			}
		} else if (cancel.equals(value)) {
			if (session.getAttribute("validUser") != null) {
				session.setAttribute("validUser", null);
			}
			if (session.getAttribute("email") != null) {
				session.setAttribute("email", null);
			}
			response.sendRedirect("login.jsp");
		} else if (logout.equals(value)) {
			if (session.getAttribute("validUser") != null) {
				session.setAttribute("validUser", null);
			}
			if (session.getAttribute("email") != null) {
				session.setAttribute("email", null);
			}
			response.sendRedirect("login.jsp");
		} else if (changepass.equals(value)) {
			response.sendRedirect("changepass.jsp?userid="
					+ URLEncoder.encode(id, "UTF-8"));
		}
		// validate xml file
		if (protocol.equalsIgnoreCase("biocase")) {
			if (validate.equals(value)) {
				session.setAttribute("xmlDoc",docStr.replaceAll("&", "&amp;"));
				docStr = docStr.replaceAll("&", "&amp;");
				session.setAttribute("notes", new String(request.getParameter("notes").toString().getBytes("ISO-8859-1"),"UTF-8"));
				Validator vd = new Validator(logger);
				vd.validate(docStr);
				vmessage = vd.getMessage();
				session.setAttribute("VMessage", vmessage);
				
				String loginURL = "editor.jsp?userid="
						+ URLEncoder.encode(id, "UTF-8") + "&Msg="
						+ URLEncoder.encode(vmessage, "UTF-8");
				response.sendRedirect(loginURL);
			}
		}
	}

	/**
	 * replace invalid characters
	 */

	private String replaceInvalidChar(String givenString) {
		givenString = givenString.replaceAll("&", "&amp;");
		givenString = givenString.replaceAll(">", "&gt;");
		givenString = givenString.replaceAll("<", "&lt;");
		givenString = givenString.replaceAll("'", "&apos;");
		givenString = givenString.replaceAll("\"", "&quot;");
		return givenString;
	}

	private String replaceInvalidCharBack(String givenString) {
		//givenString = givenString.replaceAll("&amp;","&" );
		givenString = givenString.replaceAll("&gt;",">" );
		givenString = givenString.replaceAll("&lt;","<" );
		givenString = givenString.replaceAll("&apos;","'");
		givenString = givenString.replaceAll("&quot;","\"" );
		return givenString;
	}

	
	class myFormatter extends SimpleFormatter {
		final SimpleDateFormat time_pattern = new SimpleDateFormat(
				"HH:mm:ss.SSS");

		public String format(LogRecord record) {
			return time_pattern.format(new java.util.Date(record.getMillis()))
					+ "  " + record.getLevel() + "\t" + record.getMessage()
					+ "\r\n";
		}
	}

}
