package feedback;

/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is ProcessResult.java
 *
 * The Initial Developer of the Original Code is
 * Jinling Li, Freie Universit�t Berlin, ZE Botanischer Garten und Botanisches Museum
 * Berlin-Dahlem.
 *
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.io.StringReader;

import java.sql.*;
import java.util.*;

import javax.mail.*;
import javax.mail.internet.*;

import org.tmatesoft.svn.core.SVNCommitInfo;
import org.tmatesoft.svn.core.SVNDirEntry;
import org.tmatesoft.svn.core.SVNErrorCode;
import org.tmatesoft.svn.core.SVNErrorMessage;
import org.tmatesoft.svn.core.SVNException;
import org.tmatesoft.svn.core.SVNNodeKind;
import org.tmatesoft.svn.core.SVNProperty;
import org.tmatesoft.svn.core.SVNURL;
import org.tmatesoft.svn.core.auth.ISVNAuthenticationManager;
import org.tmatesoft.svn.core.internal.io.dav.DAVRepositoryFactory;
import org.tmatesoft.svn.core.internal.io.svn.SVNRepositoryFactoryImpl; // import
// org.tmatesoft.svn.core.internal.wc.SVNConfigFile;
// import org.tmatesoft.svn.core.wc.SVNCommitClient;
import org.tmatesoft.svn.core.io.ISVNEditor;
import org.tmatesoft.svn.core.io.SVNRepository;
import org.tmatesoft.svn.core.io.SVNRepositoryFactory;
import org.tmatesoft.svn.core.io.diff.SVNDeltaGenerator;
import org.tmatesoft.svn.core.wc.SVNWCUtil;
import org.tmatesoft.svn.core.io.ISVNWorkspaceMediator; // import
// org.tmatesoft.svn.core.io.diff.SVNDiffInstruction;
// import org.tmatesoft.svn.core.io.diff.SVNDiffWindowBuilder;
// import org.tmatesoft.svn.core.io.diff.SVNDiffWindow;

import org.xml.sax.InputSource;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.Namespace;
import org.jdom.input.SAXBuilder;
import org.jaxen.JaxenException;
import org.jaxen.jdom.JDOMXPath;

import org.apache.log4j.Logger;
import org.apache.log4j.Level;

// import feedback.svn.SVNController;
// import org.tmatesoft.svn.core.wc.SVNWCClient;

/**
 * <p>
 * Title: ProcessResult
 * </p>
 * <p>
 * Description: Deal all the business related annotation
 * </p>
 * <p>
 * August, 2005
 * </p>
 * <p>
 * Company: Botanic Garden and Botanical Museum Berlin-Dahlem
 * </p>
 * 
 * @author Jinling Li
 * @version 1.0
 */

public class ProcessResult {

	private String url;
	private String protocol;
	private String standard;
	private String datasetId;
	private String sourceId;
	private String unitId;
	private int userid;
	// private int timeout = 10000; // timeout in milliseconds (for
	// HTTPConnection);
	private int wait = 200000; // time in milliseconds to wait before accessing
	// the resultlist
	private java.util.Date now;
	private Logger logger;
	private String notes = "";
	private long orig_revision = 0;
	private long this_revision = 0;
	// private String filename;
	private Config config;
	private SVNURL svnurl;
	private String uddi;
	private SVNRepository repository;
	private String SVNusername;
	private String SVNuserpassword;
	private SVNNodeKind nodeKind;
	private String SVNfeedbackurl;
	private String SVNfeedbacktrunk;
	private String SVNfeedbackbranches;
	private String newfilesvn = null;
	private String origfilesvn = null;
	private String editorEmail = null;

	public ProcessResult(Logger logger, Config config, String uddi, String url,
			String protocol, String standard, String datasetId,
			String sourceId, String unitId, int userid) {

		this.url = url;
		this.datasetId = datasetId;
		this.sourceId = sourceId;
		this.unitId = unitId;
		this.userid = userid;
		this.protocol = protocol;
		this.standard = standard;

		now = new java.util.Date();
		this.logger = logger;
		this.config = config;
		this.uddi = uddi;

		try {
			/*
			 * Initializes the library to work with a repository either via
			 * svn:// (and svn+ssh://) or via http:// (and https://) for DAV
			 * (over http and https)
			 */
			DAVRepositoryFactory.setup();
			SVNRepositoryFactoryImpl.setup();

			SVNusername = config.getSVNUsername();
			SVNuserpassword = config.getSVNPassword();
			SVNfeedbackurl = config.getSVNfeedbackurl();
			svnurl = SVNURL.parseURIEncoded(config.getSVNhost()
					+ SVNfeedbackurl);
			SVNfeedbacktrunk = config.getSVNfeedbacktrunk();
			SVNfeedbackbranches = config.getSVNfeedbackbranches();
			repository = SVNRepositoryFactory.create(svnurl);
			logger.log(Level.INFO, "SVN Repository Connection Created");
			ISVNAuthenticationManager authManager = SVNWCUtil
					.createDefaultAuthenticationManager(SVNusername,
							SVNuserpassword);
			repository.setAuthenticationManager(authManager);
			logger.log(Level.INFO,
					"SVN Repository authentification Successfull");
			nodeKind = repository.checkPath(SVNfeedbacktrunk, -1);
			if (nodeKind == SVNNodeKind.NONE) {
				SVNErrorMessage err = SVNErrorMessage.create(
						SVNErrorCode.UNKNOWN, "No entry at URL ''{0}''",
						SVNfeedbacktrunk);
				throw new SVNException(err);
			}

			nodeKind = repository.checkPath(SVNfeedbackbranches, -1);
			if (nodeKind == SVNNodeKind.NONE) {
				SVNErrorMessage err = SVNErrorMessage.create(
						SVNErrorCode.UNKNOWN, "No entry at URL ''{0}''",
						SVNfeedbackbranches);
				throw new SVNException(err);
			}
		} catch (SVNException e) {
			e.printStackTrace();
			logger.log(Level.ERROR, e.getMessage());
		}
	}

	public ProcessResult(Logger logger, int userid, String url,
			String datasetId, String sourceId, String unitId) {

		this.userid = userid;
		this.logger = logger;
		this.url = url;
		this.datasetId = datasetId;
		this.sourceId = sourceId;
		this.unitId = unitId;

		now = new java.util.Date();
	}

	public void createAnnotation(String standard, long revision, String notes,
			Config conf, String oldStr, String docStr, Connection dbConn,
			String editorEmail) {

		this.editorEmail = editorEmail;
		boolean hasNewRecord = false;
		this.notes = notes;
		// if xml is valid ( for biocase protocol) or is well formed (for digir)
		// then create a new xml file
		if (docStr.length() > 0) {

			// get the revision number
			revision = processSVN(false, docStr, revision, notes);
			hasNewRecord = true;
			// this.this_revision=revision;

		}
		// insert data into db

		insertAnnotation(dbConn);

		// emails to the biocase provider
		if (conf != null & oldStr.length() > 0 & docStr.startsWith("<?xml")) {

			emailToProvider(standard, this.origfilesvn, this.newfilesvn,
					hasNewRecord, notes, oldStr, conf);
			// emails to the digir provider.
		} else if (conf == null) {
			// ???????digir
		}
	}

	public void setOrigrevision(long revision) {
		this.orig_revision = revision;
	}

	/**
	 * 
	 * @param pathDir
	 * @param newDirName
	 * @param logcomment
	 */
	private void CheckandAddSVNDir(String pathDir, String newDirName,
			String logcomment) {
		try {
			ISVNEditor editor;
			SVNCommitInfo commitInfo = null;
			nodeKind = repository.checkPath(pathDir + "/" + newDirName, -1);
			if (nodeKind == SVNNodeKind.NONE) {
				if (svnurl.getProtocol().contains("http") == true) {
					editor = repository.getCommitEditor(logcomment,
							new CommitMediator());
				} else {
					editor = repository.getCommitEditor(logcomment, null);
				}
				editor.openRoot(-1);
				editor.openDir(pathDir, -1);
				editor.addDir(newDirName, null, -1);
				editor.closeDir();
				editor.closeDir();
				commitInfo = editor.closeEdit();
			}
		} catch (SVNException e) {
			logger.log(Level.ERROR, "Error create the " + newDirName + "in"
					+ pathDir);
			e.printStackTrace();
		}
	}

	/**
	 * 
	 * @param fileName -
	 *            xml file with provider response
	 * @param isOld -
	 *            true tells if the file is an original xml, false - edited xml
	 * @return revision number
	 */
	public long processSVN(boolean isOld, String docStr, long origrevision,
			String notes) {
		String pathSvnPrev = null;
		String pathSvn = null;
		long revision = 0;
		ISVNEditor editor;
		SVNCommitInfo commitInfo = null;
		try {
			if (isOld == true) {
				// Check the trunk/uddi (original/uddi)
				pathSvnPrev = SVNfeedbacktrunk;
				pathSvn = SVNfeedbacktrunk + "/" + uddi.trim();
				CheckandAddSVNDir(pathSvnPrev, uddi.trim(),
						"Uddi directory added");

				// // Check the trunk
				// (original/uddi/standard(digir/abcd1.2/abcd2.05/abcd2.06))
				pathSvnPrev = pathSvn;
				pathSvn = pathSvn + "/" + standard.trim();
				CheckandAddSVNDir(pathSvnPrev, standard.trim(),
						"Standard directory added");

				// Check the trunk (original/uddi/standard/sourceId(Institution)
				if (sourceId.length() > 0) {
					pathSvnPrev = pathSvn;
					pathSvn = pathSvn + "/" + sourceId.trim();
					CheckandAddSVNDir(pathSvnPrev, sourceId.trim(),
							"Institution directory added");
				}
				if (datasetId.length() > 0) {
					pathSvnPrev = pathSvn;
					pathSvn = pathSvn + "/" + datasetId.trim();
					CheckandAddSVNDir(pathSvnPrev, datasetId.trim(),
							"Collection directory added");
				}
				repository.closeSession();
				String filePath = pathSvn + "/" + unitId.trim() + ".xml";
				nodeKind = repository.checkPath(filePath, -1);

				if (nodeKind == SVNNodeKind.NONE) {
					// the original data not in the repository
					if (svnurl.getProtocol().contains("http") == true) {
						editor = repository.getCommitEditor(
								"Record xml file added", new CommitMediator());
					} else {
						editor = repository.getCommitEditor(
								"Record xml file added", null);
					}
					editor.openRoot(-1);
					byte[] data = docStr.getBytes();

					filePath = unitId.trim() + ".xml";
					editor.openDir(pathSvn, -1);
					editor.addFile(filePath, null, -1);
					editor.applyTextDelta(filePath, null);
					SVNDeltaGenerator deltaGenerator = new SVNDeltaGenerator();
					String checksum = deltaGenerator.sendDelta(filePath,
							new ByteArrayInputStream(data), editor, true);
					editor.closeFile(filePath, checksum);
					editor.closeDir();
					editor.closeDir();
					commitInfo = editor.closeEdit();
					revision = commitInfo.getNewRevision();
					logger.log(Level.INFO,
							"New record data created into SVN Repository");
				}
				if (nodeKind == SVNNodeKind.FILE) {
					// 
					String oldchecksum;
					Map fileProps = new HashMap();

					svnurl = SVNURL.parseURIEncoded(config.getSVNhost()
							+ SVNfeedbackurl + "/" + pathSvn);
					filePath = unitId.trim() + ".xml";
					SVNRepository repository1 = SVNRepositoryFactory
							.create(svnurl);
					logger.log(Level.INFO, "Repository Object Created");
					ISVNAuthenticationManager authManager = SVNWCUtil
							.createDefaultAuthenticationManager(SVNusername,
									SVNuserpassword);
					repository1.setAuthenticationManager(authManager);
					ByteArrayOutputStream xmlFileContentsStream = new ByteArrayOutputStream();
					repository1.getFile(filePath, -1, fileProps,
							xmlFileContentsStream);
					revision = SVNProperty.longValue((String) fileProps
							.get(SVNProperty.COMMITTED_REVISION));
					oldchecksum = (String) fileProps.get(SVNProperty.CHECKSUM);
					// check if the raw data is the same ...
					editor = repository1.getCommitEditor(
							"Record xml file updated", null);
					editor.openRoot(-1);
					byte[] data = docStr.getBytes();
					editor.openFile(filePath, -1);
					editor.applyTextDelta(filePath, null);
					SVNDeltaGenerator deltaGenerator = new SVNDeltaGenerator();
					String checksum = deltaGenerator.sendDelta(filePath,
							new ByteArrayInputStream(data), editor, true);
					this.origfilesvn = config.getSVNhost() + SVNfeedbackurl
							+ "/" + pathSvn + "/" + filePath;
					logger.debug("Orognal data file path:" + this.origfilesvn);
					if (oldchecksum.equalsIgnoreCase(checksum)) {
						logger.log(Level.INFO, "same checksum");
						editor.abortEdit();
						// editor.closeFile(filePath, oldchecksum);
						// editor.closeDir();
					} else {
						editor.applyTextDelta(filePath, checksum);
						editor.closeFile(filePath, checksum);
						editor.closeDir();
						commitInfo = editor.closeEdit();
						revision = commitInfo.getNewRevision();
						logger.log(Level.INFO,
								"New record data created into SVN Repository");
					}
				}
			}
		} catch (SVNException e) {
			logger.log(Level.ERROR, e.getMessage());
			e.printStackTrace();
		}

		try {
			if (isOld == false) {
				pathSvnPrev = SVNfeedbackbranches;
				pathSvn = SVNfeedbackbranches + "/" + uddi.trim();
				// Check the branches/uddi (modified/uddi)
				CheckandAddSVNDir(pathSvnPrev, uddi.trim(),
						"Uddi directory added");

				// Check the
				// branches(modified/uddi/standard(digir/abcd1.2/abcd2.05/abcd2.06))
				pathSvnPrev = pathSvn;
				pathSvn = pathSvn + "/" + standard.trim();
				CheckandAddSVNDir(pathSvnPrev, standard.trim(),
						"Standard directory added");

				if (sourceId.length() > 0) {
					pathSvnPrev = pathSvn;
					pathSvn = pathSvn + "/" + sourceId.trim();
					// Check the branches(modified/uddi/standard/sourceiD)
					CheckandAddSVNDir(pathSvnPrev, sourceId.trim(),
							"Institution directory added");

				}
				if (datasetId.length() > 0) {
					pathSvnPrev = pathSvn;
					pathSvn = pathSvn + "/" + datasetId.trim();
					// Check the branches(modified/uddi/standard/sourceiD)
					CheckandAddSVNDir(pathSvnPrev, datasetId.trim(),
							"Collection directory added");

				}
				String dirPath = pathSvn;
				String filePathtrunk = pathSvn.replaceFirst(
						SVNfeedbackbranches, SVNfeedbacktrunk)
						+ "/" + unitId.trim() + ".xml";

				Collection dircol = null;
				dircol = repository
						.getDir(dirPath, -1, null, (Collection) null);

				Integer nextID = 1;
				Iterator listiterator = dircol.iterator();
				while (listiterator.hasNext() == true) {

					if (((SVNDirEntry) listiterator.next()).getName()
							.startsWith(unitId.trim())) {
						nextID++;
					}

				}
				// Long nextID =new Long(dircol.size()+1);
				String commnum = nextID.toString();
				String filePath = pathSvn + "/" + unitId.trim() + "_" + commnum
						+ ".xml";
				// dirPath=SVNfeedbackbranches + "/" + sourceId + "/" +
				// datasetId;
				editor = repository.getCommitEditor(notes, null);
				// this.origfilesvn=config.getSVNhost()+SVNfeedbackurl+"/"+pathSvn+"/"+filePath;
				editor.openRoot(-1);
				editor.openDir(dirPath, -1);
				byte[] data = docStr.getBytes();
				editor.addFile(filePath, filePathtrunk, origrevision);
				editor.applyTextDelta(filePath, null);
				SVNDeltaGenerator deltaGenerator = new SVNDeltaGenerator();
				String checksum = deltaGenerator.sendDelta(filePath,
						new ByteArrayInputStream(data), editor, true);
				editor.closeFile(filePath, checksum);
				editor.closeDir();
				editor.closeDir();
				commitInfo = editor.closeEdit();
				this.newfilesvn = filePath;
				logger.debug("Annotated data file path:" + this.newfilesvn);
				this.this_revision = commitInfo.getNewRevision();
				revision = this_revision;
				// this.this_revision=origrevision;
				logger.log(Level.INFO,
						"New annotation created into SVN Repository");
			}
		} catch (SVNException e) {
			logger.log(Level.ERROR,
					"Error create the modified XML file into SVN");
			logger.log(Level.ERROR, e.getMessage());
			e.printStackTrace();
		}
		return revision;

	}

	private void emailToProvider(String standard, String cvsURL, String newCvs,
			boolean hasNew, String notes, String oldStr, Config conf) {

		String emailPath = "";
		Namespace ns = null;

		if (standard.equalsIgnoreCase("abcd1.2")) {
			emailPath = conf.get12EmailPath();
			ns = Namespace.getNamespace(conf.get12NS());
		} else if (standard.equalsIgnoreCase("abcd2.0")
				|| standard.equalsIgnoreCase("abcd2.05")
				|| standard.equalsIgnoreCase("abcd2.06")) {
			emailPath = conf.get20EmailPath();
			ns = Namespace.getNamespace(conf.get20NS());
		} else if (standard.equalsIgnoreCase("darwin")||standard.equalsIgnoreCase("darwincore")) {
			emailPath = conf.getDarwinEmailPath();
		}

		Vector v = getProviderEmails(oldStr, emailPath, ns);

		ProcessMail(cvsURL, newCvs, hasNew, notes, conf, v);

	}

	private Vector<String> getProviderEmails(String docStr, String xpath,
			Namespace ns) {

		Vector<String> emailV = new Vector<String>();
		// remove the namespace of the
		// datasets.(JDOMXPath(xpath).selectNodes(doc) can not
		// work properly with the namespace). try to find another way later.
		int index1 = docStr.indexOf("xmlns");
		int index2 = docStr.indexOf("<DataSet>");

		if (index1 > 0 && index2 > 0) {
			docStr = docStr.substring(0, index1 - 1) + ">"
					+ docStr.substring(index2);

			try {
				SAXBuilder builder = new SAXBuilder();
				Document doc = builder.build(new InputSource(new StringReader(
						docStr)));
				ArrayList<Element> emails = (ArrayList<Element>) (new JDOMXPath(
						xpath).selectNodes(doc));
				Iterator<Element> i = (Iterator<Element>) emails.iterator();
				while (i.hasNext()) {
					String emailString = ((Element) i.next()).getText();

					// [at]
					int indexat = emailString.indexOf("[at]");
					if (indexat > 0) {
						int atsur1 = emailString.indexOf("[");
						int atsur2 = emailString.indexOf("]");
						if (atsur1 > 0 && atsur2 > 0) {
							emailString = emailString.replace(emailString
									.substring(atsur1, atsur2 + 1), "@");
						}
						;
					}
					;
					logger.debug("Contact Email :" + emailString);
					emailV.add(emailString);
					logger.debug("got contact emails");
				}
			} catch (Exception e) {
				e.printStackTrace();
				
			}
		} else {
			logger.info("Digir provider . Querying metadata.");
			String resourcename = url.substring(0,url.indexOf("@"));
			url = url.substring(url.indexOf("@")+1);
			MetadataQuery mquery = new MetadataQuery(this.logger, url, 20000);
			String metadata = mquery.getMetadata();
			int index1d = metadata.indexOf("xmlns");
			int index2d = metadata.indexOf("<header>");
			metadata = metadata.substring(0, index1d - 1) + ">" + metadata.substring(index2d);
			SAXBuilder builder = new SAXBuilder();
			try {
				Document doc = builder.build(new InputSource(new StringReader(
						metadata)));
				String xpathresource = "//resource";
				ArrayList<Element> resources = (ArrayList<Element>) (new JDOMXPath(
						xpathresource).selectNodes(doc));
				Iterator<Element> it_resources = resources.iterator();
				boolean foundres = false;
				while (it_resources.hasNext() && !foundres) {
					Element resource = it_resources.next();
					if (resource.getChild("code").getValue().equalsIgnoreCase(
							resourcename)) {
						foundres = true;
						logger.debug("found resource: "+resource.getText());
						List<Element> contactlist = (List<Element>) resource
								.getChildren("contact");
						Iterator<Element> iterator_contacts = contactlist
								.iterator();
						while (iterator_contacts.hasNext()) {
							Element contact = iterator_contacts.next();
							if (contact.getAttribute("type").getValue()
									.equalsIgnoreCase("administrative")) {
								logger.debug("found contact:" + contact.getText());
								Element email = contact
										.getChild("emailAddress");
								logger.debug("found email address : "+email.getValue());
								String emailString = email.getValue();
								emailV.add(emailString);
								logger.debug("Contact Email :" + emailString);
							}
						}
					}
				}
			} catch (JDOMException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (JaxenException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		return emailV;
	}

	private void ProcessMail(String cvsURL, String newCvs, boolean hasNew,
			String notes, Config conf, Vector<String> v) {

		String subject = conf.getAnnotationEmailSubject();
		String content = conf.getAnnotationEmailContent();
		content = content.replaceFirst("__notes__", this.notes);
		if (notes == null || notes.length() == 0) {
			content = content.replaceFirst("Comment : ", "");
		}
		boolean filesOK = false;
		String replacement = origfilesvn;
		if (replacement != null) {
			content = content.replaceAll("__origCVS__", replacement);
			filesOK = true;
		}
		replacement = conf.getSVNhost() + config.getSVNfeedbackurl()
				+ newfilesvn;
		if (replacement != null && filesOK == true) {
			content = content.replaceAll("__newCvs__", replacement);
		}
		if (filesOK == true) {
			Enumeration<String> e = v.elements();
			while (e.hasMoreElements()) {

				String email = e.nextElement().toString();
				sendEmail(conf, email, subject, content);
				logger.debug("Email message succesfully sent to : " + email);
			}
		}
	}

	private void sendEmail(Config conf, String email, String subject,
			String content) {

		try {
			String host = conf.getHost();
			String from = conf.getSender();
			String editoremail = this.editorEmail;

			Properties props = new Properties();
			// set the host for sending email
			props.put("mail.host", host);

			Session mailConnection = Session.getInstance(props);

			Message msg = new MimeMessage(mailConnection);
			Address biocase = new InternetAddress(from, "Feedback");
			Address recipient = new InternetAddress(email);
			Address editor = new InternetAddress(editoremail);

			msg.setFrom(biocase);
			msg.setRecipient(Message.RecipientType.TO, recipient);
			msg.addRecipient(Message.RecipientType.CC, editor);
			msg.setSubject(subject);
			msg.setContent(content, "text/html");
			Transport.send(msg);
			logger.info("finished to send email to " + email + " and "
					+ editoremail);

		} catch (Exception e) {
			e.printStackTrace();
		}

	}

	private void insertAnnotation(Connection dbConn) {
		int annotationid = nextFreeId("annotation", dbConn);
		String sql = "INSERT into annotation(" + "annotation_id, "
				+ "user_fk, " + "annotation_date, " + "notes, "
				+ "orig_revision, " + "this_revision , " + "service_url, "
				+ "uddi_key, " + "inst_id, " + "coll_id, " + "unit_id, "
				+ "filename," + "format"
				+ ") values (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)";
		try {
			PreparedStatement pstmt = null;
			pstmt = dbConn.prepareStatement(sql);

			pstmt.setInt(1, annotationid);
			pstmt.setInt(2, userid);
			pstmt.setTimestamp(3, new Timestamp(now.getTime()));

			if (notes.equals("")) {
				pstmt.setNull(4, Types.VARCHAR);
			} else {
				pstmt.setString(4, notes);
			}

			if (this.orig_revision == 0) {
				pstmt.setNull(5, Types.VARCHAR);
			} else {
				pstmt.setLong(5, this.orig_revision);
			}

			if (this.this_revision == 0) {
				pstmt.setNull(6, Types.VARCHAR);
			} else {
				pstmt.setLong(6, this.this_revision);
			}

			if (url.equals("")) {
				pstmt.setNull(7, Types.VARCHAR);
			} else {
				pstmt.setString(7, url);
			}

			if (uddi.equals("")) {
				pstmt.setNull(8, Types.VARCHAR);
			} else {
				pstmt.setString(8, this.uddi.trim());
			}

			if (sourceId.equals("")) {
				pstmt.setNull(9, Types.VARCHAR);
			} else {
				pstmt.setString(9, sourceId);
			}

			if (datasetId.equals("")) {
				pstmt.setNull(10, Types.VARCHAR);
			} else {
				pstmt.setString(10, datasetId);
			}
			if (unitId == null) {
				unitId = "";
			}
			;
			if (unitId.equals("")) {
				pstmt.setNull(11, Types.VARCHAR);
			} else {
				pstmt.setString(11, unitId);
			}

			if (this.newfilesvn.equals("")) {
				pstmt.setNull(12, Types.VARCHAR);
			} else {
				pstmt.setString(12, config.getSVNhost()
						+ config.getSVNfeedbackurl() + this.newfilesvn);
			}
			String fullnamespace = null;
			if (standard.equalsIgnoreCase("abcd1.2") == true) {
				fullnamespace = config.get12NS();
			}
			if (standard.equalsIgnoreCase("abcd2.0") == true) {
				fullnamespace = config.get20NS();
			}
			if (standard.equalsIgnoreCase("abcd2.06") == true) {
				fullnamespace = config.get206NS();
			}
			if (standard.equalsIgnoreCase("abcd2.05") == true) {
				fullnamespace = config.get205NS();
			}
			if (standard.equalsIgnoreCase("darwincore") == true) {
				fullnamespace = config.getdigirNS();
			}
			if (standard.equalsIgnoreCase("digir_manis") == true) {
				fullnamespace = config.getdigirmanisNS();
			}
			if (standard.equalsIgnoreCase("digir_obis") == true) {
				fullnamespace = config.getdigirobisNS();
			}
			pstmt.setString(13, fullnamespace.trim());

			pstmt.executeUpdate();
			pstmt.close();
			pstmt = null;

			logger.log(Level.INFO, "New annotation created into Database");
			// System.out.println("annotation information has been saved!");

		} catch (SQLException e) {
			// System.out.println();
			logger.log(Level.ERROR,
					"SQL exception when inserting a new annotation!");
			e.printStackTrace();
		}
	}

	/**
	 * Method to access an intermediate resultset waiting a few seconds and
	 * returning what has been returned.
	 */
	public String getIntermediate() {

		QueryUnit qu = new QueryUnit(logger, url, protocol, standard,
				datasetId, sourceId, unitId, config);

		qu.SendQuery();

		// try to get some intermediate results ...
		String docStr = qu.getDocStr();
		int sec = 0;
		while (docStr.length() == 0 & ((sec * 1000) < this.wait)) {
			// wait until waiting time is over or a sufficient result set has
			// been achieved ...
			try {

				Thread.sleep(1000); // wait one second
				sec++;
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
			// ... and get some results
			docStr = qu.getDocStr();
			logger.debug("Original XML:\n" + docStr);
		}
		return docStr;
	}

	private String getFileDir(Config conf) {

		String separator = System.getProperty("file.separator");

		String fileDir = conf.getRawDataDir();

		// fileDir = fileDir + separator + "source_docs" + separator
		// + url.replaceAll("http://", "").replaceAll("/", "_")
		// + separator + sourceId + separator + datasetId + separator;

		fileDir = fileDir + separator + "source_docs" + separator;
		// check if directory already exists

		File Fdir = new File(fileDir);

		if (!Fdir.isDirectory()) {
			Fdir.mkdirs();

		}

		return fileDir;
	}

	// create the filename for a raw data xml file
	private String getRawdataFilename(Config conf) {

		// check if directory already exists

		String dir = getFileDir(conf);

		String filename = "";

		filename = dir + unitId + ".xml";

		return filename;
	}

	public String createNNfile(Config conf, String docStr) {

		String fileName = getRawdataFilename(conf);
		// createNNfile(docStr, fileName, true);

		return fileName;
	}

	public String createNNfile(String docStr, String oldname, boolean isOld) {

		File file = null;
		String dir = "";
		String filename = "";
		if (isOld == true) {
			// String string = oldname = oldname;
			file = new File(oldname);
			System.out.println("oldfile:" + file.getAbsolutePath());
		} else {
			oldname = oldname.replaceFirst("source_docs", "commented_docs");
			int fileIndex = oldname.lastIndexOf(System
					.getProperty("file.separator"));

			dir = oldname.substring(0, fileIndex + 1);
			filename = oldname.substring(fileIndex + 1);
			File outputFile = new File(dir);
			if (!outputFile.isDirectory()) {
				outputFile.mkdirs();

			}

			file = new File(dir + filename);
			System.out.println("newfile:" + file.getAbsolutePath());

		}

		StringBuffer buffer;
		buffer = new StringBuffer();
		buffer.append(docStr);

		String finalfile = buffer.toString();

		try {
			StringReader read = new StringReader(finalfile);

			Writer osw = new OutputStreamWriter(new FileOutputStream(file),
					"UTF8");

			int ch;
			while ((ch = read.read()) != -1) {

				osw.write((char) ch);

			}

			osw.close();
			System.out.println("xml file saved.");
		} catch (IOException e) {
			// System.out.println("--> IOException " + oldname + " could not be
			// created.");
			System.out.println("--> IOException " + e);
		}
		return file.toString();

	}

	// Return the first unoccupied identifier (integer) in the given table.
	private int nextFreeId(String tableName, Connection dbConn) {
		int newId = -1;
		String theQuery = "";

		// Call user postgres user defined functions to determin new ids
		if (tableName.equals("annotation")) {
			theQuery = "SELECT newannotationid()";

		}

		try {
			Statement stmt1 = null;
			stmt1 = dbConn.createStatement();
			ResultSet idRs = null;
			idRs = stmt1.executeQuery(theQuery);
			idRs.next();
			newId = idRs.getInt(1);
			stmt1.close();
			stmt1 = null;
			idRs.close();
			idRs = null;
		} catch (SQLException e) {
			System.out
					.println("Exception when trying to generate a new id for table: "
							+ tableName);
			e.printStackTrace();
		} finally {
			return newId;
		}
	}

	public class CommitMediator implements ISVNWorkspaceMediator {
		private Map myTmpStorages = new HashMap();

		/*
		 * This may be implemented to get properties from '.svn/wcprops'
		 */
		public String getWorkspaceProperty(String path, String name)
				throws SVNException {
			return null;
		}

		/*
		 * This may be implemented to set properties in '.svn/wcprops'
		 */
		public void setWorkspaceProperty(String path, String name, String value)
				throws SVNException {
		}

		/*
		 * Creates a temporary file delta storage. id will be used as the
		 * temporary storage identifier. Returns an OutputStream to write the
		 * delta data into the temporary storage.
		 */
		public OutputStream createTemporaryLocation(String path, Object id) {
			ByteArrayOutputStream tempStorageOS = new ByteArrayOutputStream();
			myTmpStorages.put(id, tempStorageOS);
			return tempStorageOS;
		}

		/*
		 * Returns an InputStream of the temporary file delta storage identified
		 * by id to read the delta.
		 */
		public InputStream getTemporaryLocation(Object id) {
			return new ByteArrayInputStream(
					((ByteArrayOutputStream) myTmpStorages.get(id))
							.toByteArray());
		}

		/*
		 * Gets the length of the delta that was written to the temporary
		 * storage identified by id.
		 */
		public long getLength(Object id) {
			ByteArrayOutputStream tempStorageOS = (ByteArrayOutputStream) myTmpStorages
					.get(id);
			if (tempStorageOS != null) {
				return tempStorageOS.size();
			}
			return 0;
		}

		/*
		 * Deletes the temporary file delta storage identified by id.
		 */
		public void deleteTemporaryLocation(Object id) {
			myTmpStorages.remove(id);
		}
	}
}