package feedback;

/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is UnitPoster.java
 *
 * The Initial Developer of the Original Code is
 * Jinling Li, Freie Universit�t Berlin, ZE Botanischer Garten und Botanisches Museum
 * Berlin-Dahlem .
 *
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */

import java.io.IOException;
import java.io.InputStream;
import java.io.StringBufferInputStream;
import java.io.StringWriter;
import java.io.Writer;
import java.net.MalformedURLException;
import java.net.URL;
import java.nio.Buffer;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.jdom.Document;
import org.jdom.JDOMException;
import org.jdom.Text;
import org.jdom.input.DOMBuilder;
import org.jdom.input.SAXBuilder;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;

import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;

import HTTPClient.CookieModule;
import HTTPClient.HTTPConnection;
import HTTPClient.HTTPResponse;
import HTTPClient.ModuleException;
import HTTPClient.NVPair;

/**
 * <p>
 * Title: UnitPoster
 * </p>
 * <p>
 * Description: class to get XML file from given URL
 * </p>
 * <p>
 * Company: Botanic Garden and Botanical Museum Berlin-Dahlem
 * </p>
 * 
 * @author Jinling Li
 * @version 1.0
 */

public class UnitPoster extends Thread {
	// private int bodyid;
	private String protocol;
	private QueryUnit callback;
	private String theQuery;
	private URL url; // full url of the unit level wrapper software
	private String urlString;
	private String cgiScriptPath; // path to the wrapper url within the domain
	private int timeout;
	private Config conf; // timeout in milliseconds
	private String rootElement;
	private Logger logger;

	public UnitPoster(Logger logger, String theQuery, String url, int timeout,
			Config conf, String protocol, String rootElement, QueryUnit callback) {
		super();
		// convert the url string
		this.callback = callback;
		this.theQuery = theQuery;
		this.timeout = timeout;
		this.conf = conf;
		this.protocol = protocol;
		this.rootElement = rootElement;
		this.logger = logger;
		CookieModule.setCookiePolicyHandler(null);
		try {

			this.url = new URL(url);
			cgiScriptPath = this.url.getPath();
			if (this.url.getQuery() != null) {
				cgiScriptPath = cgiScriptPath + "?" + this.url.getQuery();
			}
			int index = (this.url.toString()).lastIndexOf(cgiScriptPath);
			urlString = this.url.toString().substring(0, index);
		} catch (MalformedURLException e) {
			System.out.println("URL seems to be malformed (" + url + ").");

			e.printStackTrace();
		}

	}

	public void run() {
		StringBuffer sb = new StringBuffer();
		try {

			HTTPConnection con = new HTTPConnection(new URL(urlString));
			con.setTimeout(this.timeout);
			NVPair queryData[] = new NVPair[1];
			
			if (protocol.equalsIgnoreCase("biocase")) {
				queryData[0] = new NVPair("query", this.theQuery);
			}
			if (protocol.equalsIgnoreCase("digir")) {
				queryData[0] = new NVPair("request", this.theQuery);
			}
			HTTPResponse rsp = con.Post(cgiScriptPath, queryData);
			// HTTPResponse rsp = con.Get(cgiScriptPath, queryData);
			if (rsp.getStatusCode() >= 300) {
				// some logging should be implemented here
				// System.out.println("connection Error");
				logger.error( " HTTP connection Error 300");
			} else {

				InputStream in = rsp.getInputStream();
				
				if (!isInterrupted()) {
					try {
						// Reader reader;
						logger.info("HTPP response OK from provider");
						InputSource ins;
						XMLReader xrp;
						ins = new InputSource(in);
						xrp = new org.apache.xerces.parsers.SAXParser();
						xrp.setFeature(
										"http://apache.org/xml/features/validation/schema",
										true);
						xrp.setFeature(
										"http://xml.org/sax/features/namespace-prefixes",
										true);

						// set the content and error handler.
						XmlExtractor xe = new XmlExtractor(logger, conf,
								protocol);
						xrp.setContentHandler(xe);
						xrp.setErrorHandler(xe);
						xrp.parse(ins);
						String docStr = xe.getDocStr();

						if (docStr.length() > 0) {
							callback.setDocStr(docStr, rootElement);
						} else {
							callback
									.setDocStr(
											"Sorry, due to some problem, you can not edit the record now."
													+ "\r\n"
													+ "Please try again later or just give your comment in the notes field."
													+ "\r\n" + "thank you.",
											rootElement);
						}
					} catch (SAXException saxe) {
						// System.out.println("*** Exeption when using Sax to
						// read doc");
						logger.log(Level.ERROR,
								"*** Exeption when using Sax to read doc");
						// saxe.printStackTrace();
					}
				}
			}
		} catch (java.net.SocketTimeoutException e) {
			logger.info("Timeout");
			logger.info(e.getMessage());
			//callback. set message
		} catch (IOException ioe) {
			// System.out.println("*** IO exception when posting request to");
			logger.log(Level.ERROR, "*** IO exception when posting request to");
			ioe.printStackTrace();
		} catch (ModuleException me) {
			// System.out.println("**** Module exception when posting request to
			// ");
			logger.log(Level.ERROR,
					"**** Module exception when posting request to ");
			// me.printStackTrace();
		}
	}// end of run

}